// A simple Arduino Leonardo based program to use two rotary encoders with push buttons and three additional buttons to operate XCSoar.
// This tutorial was very helpful: http://www.loiph.in/2014/09/arduino-leonardo-atmega32u4-based-usb.html
// Modified by Stefan Langer; www.stefly.aero

#include <Encoder.h>
// The rotary encoder library http://www.pjrc.com/teensy/td_libs_Encoder.html

#include <Keyboard.h>

int accel = 2;
// This is an acceleration factor. Use between 1 and 8 to suit how quickly the volume goes up or down.

int LE_PB = 8;
int RE_PB = 7;
int Vario_PB = 9;
int free1 = 10;  // Not used at the moment, but connected with the RJ11 PIN 4
int free2 = 14;  // Not used at the moment, but connected with the RJ11 PIN 5
// The digital pin to which the rotary encoder push button is connected too. Assumes that the pin has a pull-down resistor.

int Fn = 16;
int ESC = 15;
int debounce_delay = 200;

long longpresstime = 500;

long ESCtimer = 0;
long Fntimer = 0;
long LE_PBtimer = 0;
long RE_PBtimer = 0;
long Vario_PBtimer = 0;

bool oldStatus = true;  // alter Schalterstatus HIGH = nicht gedrückt

bool ESCactive = false;
bool Fnactive = false;
bool LE_PBactive = false;
bool RE_PBactive = false;
bool Vario_PBactive = false;

bool ESCLongpressactive = false;
bool FnLongpressactive = false;
bool LE_PBLongpressactive = false;
bool RE_PBLongpressactive = false;
bool Vario_PBLongpressactive = false;

Encoder R_Enc(1, 0);

// These are the pins to which the rotary encoder is connected.
// Pins 2,3 are the interrupt pins on a Leonardo/Uno, which give best performance with a rotary encoder.
// Use other pins if you wish, but performance may suffer.
// Avoid using pins that have LED's attached.

long oldPosition  = -999;

Encoder L_Enc(3, 2);
long oldPosition2  = -999;

/*
  Encoder Vario_Enc(6, 5);
  long oldPosition3  = -999;
*/
void setup() {
  pinMode(LE_PB, INPUT_PULLUP);
  pinMode(RE_PB, INPUT_PULLUP);
  pinMode(Vario_PB, INPUT_PULLUP);
  pinMode(free1, INPUT_PULLUP);
  pinMode(free2, INPUT_PULLUP);
  pinMode(ESC, INPUT_PULLUP);
  pinMode(Fn, INPUT_PULLUP);

  Serial.begin(9600);
  //Serial.println("Debug Output:");
}

void loop() {

  //rechter Encoder
    long newPosition = R_Enc.read() / accel;
    if (newPosition != oldPosition && RE_PBactive == false) {

     if (newPosition > oldPosition) {
       Keyboard.write(KEY_DOWN_ARROW);  //Schnittstelle zu xci-file; ASCII 0x32
     }

     if (oldPosition > newPosition) {
       Keyboard.write(KEY_UP_ARROW);  //ASCII 0x31
     }

     oldPosition = newPosition;
     //Serial.println(newPosition);

    }

    if (newPosition != oldPosition && RE_PBactive == true) {

     if (newPosition > oldPosition) {
       Keyboard.write(KEY_F11);             // Volume Down
       RE_PBtimer = millis();
       delay(debounce_delay);
     }

     if (oldPosition > newPosition) {
       Keyboard.write(KEY_F12);             // Volume Up
       RE_PBtimer = millis();
       delay(debounce_delay);
     }

     oldPosition = newPosition;
     //Serial.println(newPosition);
    }


  // linker Encoder
  long newPosition2 = L_Enc.read() / accel;
  if (newPosition2 != oldPosition2 && LE_PBactive == false) {

    if (newPosition2 > oldPosition2) {
      Keyboard.write(KEY_LEFT_ARROW);  //Schnittstelle zu xci-file; ASCII 0x32
      delay(debounce_delay);
    }

    if (oldPosition2 > newPosition2) {
      Keyboard.write(KEY_RIGHT_ARROW);  //ASCII 0x31
      delay(debounce_delay);
    }

    oldPosition2 = newPosition2;
    //Serial.println(newPosition2);

  }

  //
  // Die folgenden Zeilen aktivieren, wenn Drücken und Drehen beim
  // linken Encoder aktiviert werden soll
  //

  /*
      if (newPosition2 != oldPosition2 && LE_PBactive == true) {

        if (newPosition2 > oldPosition2) {
          Keyboard.write(KEY_F11);             // Volume Down
          LE_PBtimer = millis();
          delay(debounce_delay);
        }

        if (oldPosition2 > newPosition2) {
          Keyboard.write(KEY_F12);             // Volume Up
          LE_PBtimer = millis();
          delay(debounce_delay);
        }

        oldPosition2 = newPosition2;
        Serial.println(newPosition2);
      }
  */

  // weiterer Encoder
  /*    long newPosition3 = Vario_Enc.read() / accel;
      if (newPosition3 != oldPosition3 && Vario_PBactive == false) {

        if (newPosition3 > oldPosition3) {
          Keyboard.write('k');  //Schnittstelle zu xci-file; ASCII 0x32
          delay(debounce_delay);
        }

        if (oldPosition3 > newPosition3) {
          Keyboard.write('l');  //ASCII 0x31
          delay(debounce_delay);
        }

        oldPosition3 = newPosition3;
        Serial.println(newPosition3);

      }

      // Die folgenden Zeilen aktivieren, wenn Drücken und Drehen beim
      // Vario-Encoder aktiviert werden soll, ansonsten deaktivieren


      if (newPosition3 != oldPosition3 && Vario_PBactive == true) {

        if (newPosition3 > oldPosition3) {
          Keyboard.write(KEY_F11);             // Volume Down
          Vario_PBtimer = millis();
          delay(debounce_delay);
        }

        if (oldPosition3 > newPosition3) {
          Keyboard.write(KEY_F12);             // Volume Up
          Vario_PBtimer = millis();
          delay(debounce_delay);
        }

        oldPosition3 = newPosition3;
        Serial.println(newPosition3);
      }
  */

  //ESC
  if (digitalRead(ESC) == LOW) {
    if (ESCactive == false) {
      ESCactive = true;
      ESCtimer = millis();
    }
    if ((millis() - ESCtimer > longpresstime) && (ESCLongpressactive == false)) {
      ESCLongpressactive = true;
      //Serial.println("Pan Mode");
      Keyboard.write(KEY_F9);  // F9
      delay(debounce_delay);
    }
  }
  else {
    if (ESCactive == true) {
      if (ESCLongpressactive == true) {
        ESCLongpressactive = false;
      }
      else {
        //Serial.println("ESC");
        Keyboard.write(KEY_ESC);  // ESC
        delay(debounce_delay);
      }
      ESCactive = false;
    }
  }

  //Fn
  if (digitalRead(Fn) == LOW) {
    if (Fnactive == false) {
      Fnactive = true;
      Fntimer = millis();
    }
    if ((millis() - Fntimer > longpresstime) && (FnLongpressactive == false)) {
      FnLongpressactive = true;
      //Serial.println("Flarm Radar");
      Keyboard.write(KEY_F3);  // F3
      delay(debounce_delay);
    }
  }
  else {
    if (Fnactive == true) {
      if (FnLongpressactive == true) {
        FnLongpressactive = false;
      }
      else {
        //Serial.println("Waypoint select");
        Keyboard.write(KEY_F5);  // F5
        delay(debounce_delay);
      }
      Fnactive = false;
    }
  }

  //LE_PB
  if (digitalRead(LE_PB) == LOW) {
    if (LE_PBactive == false) {
      LE_PBactive = true;
      LE_PBtimer = millis();
    }
    if ((millis() - LE_PBtimer > longpresstime) && (LE_PBLongpressactive == false)) {
      LE_PBLongpressactive = true;
      //Serial.println("Analysis");
      Keyboard.write(KEY_F2);  // F2
      delay(debounce_delay);
    }
  }
  else {
    if (LE_PBactive == true) {
      if (LE_PBLongpressactive == true) {
        LE_PBLongpressactive = false;
      }
      else {
        //Serial.println("Setup Alternates");
        Keyboard.write(KEY_F6);  // F6
        delay(debounce_delay);
      }
      LE_PBactive = false;
    }
  }

  //RE_PB
  if (digitalRead(RE_PB) == LOW) {
    if (RE_PBactive == false) {
      RE_PBactive = true;
      RE_PBtimer = millis();
    }
    if ((millis() - RE_PBtimer > longpresstime) && (RE_PBLongpressactive == false)) {
      RE_PBLongpressactive = true;
      //Serial.println("Quick Menue");
      Keyboard.write(KEY_F1);  // F1
      delay(debounce_delay);
    }
  }
  else {
    if (RE_PBactive == true) {
      if (RE_PBLongpressactive == true) {
        RE_PBLongpressactive = false;
      }
      else {
        //Serial.println("Return");
        Keyboard.write(KEY_RETURN);  // return
        delay(debounce_delay);
      }
      RE_PBactive = false;
    }
  }

  //weiterer PB
  /*  if (digitalRead(Vario_PB) == LOW) {
      if (Vario_PBactive == false) {
        Vario_PBactive = true;
        Vario_PBtimer = millis();
      }
      if ((millis() - Vario_PBtimer > longpresstime) && (Vario_PBLongpressactive == false)) {
        Vario_PBLongpressactive = true;
        //Serial.println("Setup Basic");
        Keyboard.write(KEY_F8);  // F8
        delay(debounce_delay);
      }
    }
    else {
      if (Vario_PBactive == true) {
        if (Vario_PBLongpressactive == true) {
          Vario_PBLongpressactive = false;
        }
        else {
          //Serial.println("McCready auto");
          Keyboard.write(KEY_F10);  // F10
          delay(debounce_delay);
        }
        Vario_PBactive = false;
      }
    }
  */
}
